import { useState, useEffect } from 'react';
import {
  Drawer,
  Button,
  IconButton,
  Toolbar,
  Box,
  TextField,
  Divider,
  Stack,
  Typography,
} from '@mui/material';
import CloseIcon from '@mui/icons-material/Close';
import { useAuth } from '../Contexts/AuthContext';
import { useSnackbar } from '../Contexts/SnackbarContext';
import { useAppContext } from '../Contexts/AppContext';

export const UserProfile = ({ showProfile, setShowProfile }) => {
  const { user, signOut } = useAuth();
  const { userSettings, updateProfile } = useAppContext();
  const { showSnackBar } = useSnackbar();
  const [profileInputs, setProfileInputs] = useState({ userSettings });

  useEffect(() => {
    setProfileInputs({ username: userSettings?.username });
  }, [showProfile, userSettings]);

  const handleChange = ({ target: { name, value } }) => {
    setProfileInputs({ ...profileInputs, [name]: value });
  };

  const handleUpdate = async (e) => {
    e.preventDefault();
    if (profileInputs.username !== '') {
      const res = await updateProfile(profileInputs.username);

      if (res) {
        showSnackBar('success', 'nickname updated');
        setShowProfile(false);
      }
    }
  };

  const handleSignOut = (e) => {
    e.preventDefault();
    signOut();
    showSnackBar('info', 'user logged out');
  };

  return (
    <Drawer
      PaperProps={{
        sx: {
          width: 280,
        },
      }}
      anchor="right"
      open={showProfile}
      onClose={() => setShowProfile(false)}
    >
      <Toolbar
        disableGutters
        variant="dense"
        sx={{ flexDirection: 'row', mx: 1 }}
      >
        <Box sx={{ flexGrow: 1 }}>
          {' '}
          <Typography
            variant="subtitle2"
            component="div"
            sx={{ flexGrow: 1, textTransform: 'uppercase' }}
          >
            user profile
          </Typography>
        </Box>
        <IconButton
          edge="end"
          size="small"
          onClick={() => setShowProfile(false)}
        >
          <CloseIcon />
        </IconButton>
      </Toolbar>
      <Divider />
      <Stack height={1} m={2} spacing={2} component="form">
        <TextField
          name="username"
          label="username"
          value={profileInputs.username}
          type="text"
          onChange={handleChange}
          required
          autoComplete="username"
        />
        {/** TODO: add update email fnc*/}
        <TextField fullWidth label="email" value={user.email} disabled />
        <Button type="submit" fullWidth onClick={(e) => handleUpdate(e)}>
          update profile
        </Button>
        <Divider sx={{ flex: 1 }} />
        <Button fullWidth onClick={(e) => handleSignOut(e)}>
          signout
        </Button>
      </Stack>
    </Drawer>
  );
};
